<?php

declare(strict_types=1);

namespace Drupal\opc\Form;

use Drupal\Component\Render\FormattableMarkup as FM;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup as TM;
use Drupal\opc\PreloadGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides a OPCache form.
 */
final class PreloadForm extends FormBase {

  /**
   * {@selfdoc}
   */
  public function __construct(private readonly PreloadGenerator $preloadGenerator) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self($container->get('opc.preload_generator'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'opc_preload';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $help = <<< 'HTML'
      <p>PHP can be configured to preload scripts into the opcache when the engine starts.
      Any functions, classes, interfaces, or traits (but not constants) in those files will
      then become globally available for all requests without needing to be explicitly included.<br/>
      Check out <a href="https://www.php.net/manual/en/opcache.preloading.php">preloading documentation</a> for details.
      </p>
      HTML;

    $form['help'] = [
      '#markup' => new FM($help, []),
    ];

    $form['memory_limit'] = [
      '#type' => 'number',
      '#title' => new TM('Memory limit'),
      '#field_suffix' => new TM('MB'),
      '#min' => 0,
      '#required' => TRUE,
      '#description' => new TM('The number of preloaded scripts will be constrained to fit this limit.'),
      '#default_value' => 64,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#button_type' => 'primary',
        '#value' => new TM('Generate preload script'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $memory_limit = $form_state->getValue('memory_limit');
    $script = $this->preloadGenerator->generatePreload(
      $memory_limit ? (int) $memory_limit : NULL,
    );

    $headers = [
      'Content-Disposition' => 'attachment; filename="preload.php"',
      'Content-type' => 'application/x-httpd-php',
    ];
    $response = new Response($script, headers: $headers);
    $form_state->setResponse($response);
  }

}
