<?php

declare(strict_types=1);

namespace Drupal\opc;

use Composer\ClassMapGenerator\ClassMapGenerator;
use Composer\InstalledVersions;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\opc\Contract\OpcacheInterface;

/**
 * {@selfdoc}
 */
final readonly class PreloadGenerator {

  /**
   * {@selfdoc}
   */
  public function __construct(
    private ModuleHandlerInterface $moduleHandler,
    private TwigEnvironment $twig,
    private ClassMapGenerator $classMapGenerator,
    private TimeInterface $time,
    private OpcacheInterface $opcache,
  ) {}

  /**
   * Generates preload script.
   */
  public function generatePreload(?int $memory_limit): string {
    $template = \file_get_contents(__DIR__ . '/../templates/preload.php.twig');
    $context = [
      'class_loader' => self::getVendorDir() . '/autoload.php',
      'generated' => $this->time->getCurrentTime(),
      'log' => 'php://output',
      'symbols' => $this->getSymbols($memory_limit),
      'files' => $this->getFiles(),
      'namespaces' => $this->getNameSpaces(),
    ];
    // @todo Invoke alter hook here.
    return (string) $this->twig->renderInline($template, $context);
  }

  /**
   * {@selfdoc}
   */
  private function getNameSpaces(): array {
    $namespaces = [];
    foreach ($this->moduleHandler->getModuleList() as $module_name => $extension) {
      $namespaces['Drupal\\' . $module_name . '\\\\'] = \DRUPAL_ROOT . '/' . $extension->getPath() . '/src';
    }
    return $namespaces;
  }

  /**
   * {@selfdoc}
   */
  private function getFiles(): array {
    $files = [];
    foreach ($this->moduleHandler->getModuleList() as $extension) {
      $file = $extension->getExtensionPathname();
      if ($file) {
        $files[] = \DRUPAL_ROOT . '/' . $file;
      }
    }
    return $files;
  }

  /**
   * {@selfdoc}
   */
  private function getSymbols(?int $memory_limit): array {
    $this->classMapGenerator->scanPaths(self::getVendorDir());
    $this->classMapGenerator->scanPaths(\DRUPAL_ROOT . '/core');
    $this->classMapGenerator->scanPaths(\DRUPAL_ROOT . '/modules');
    $class_map = $this->classMapGenerator->getClassMap();

    $scripts = $this->opcache->getStatus()['scripts'];
    $comparator = static fn (array $a, array $b): int => $b['hits'] <=> $a['hits'];
    \usort($scripts, $comparator);

    if ($memory_limit === NULL) {
      $filtered_scripts = $scripts;
    }
    else {
      $filtered_scripts = [];
      $memory_consumption = 0;
      foreach ($scripts as $script) {
        $memory_consumption += $script['memory_consumption'];
        if ($memory_consumption > ($memory_limit * 1_024 * 1_024)) {
          break;
        }
        $filtered_scripts[] = $script;
      }
    }

    $op_files = \array_column($filtered_scripts, 'full_path');
    return \array_keys(\array_intersect($class_map->getMap(), $op_files));
  }

  /**
   * {@selfdoc}
   */
  private static function getVendorDir(): string {
    $composer_path = \realpath(InstalledVersions::getInstallPath('composer/composer'));
    return \explode('/composer/composer', $composer_path, 2)[0];
  }

}
